"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqplib_1 = require("amqplib");
const connection_1 = require("amqplib/lib/connection");
const RabbitmqConfig_1 = require("./RabbitmqConfig");
class RabbitmqClient {
    constructor(clientConfig, name) {
        this.closed = false;
        clientConfig.protocol = clientConfig.protocol || 'amqp';
        this.clientConfig = clientConfig;
        this.name = name;
        this.channelNotReady();
    }
    async init() {
        await this.connect();
    }
    async awaitChannelReady() {
        if (this.channelReadyPromise) {
            await this.channelReadyPromise;
        }
    }
    channelNotReady() {
        if (!this.channelReadyPromise) {
            this.channelReadyPromise = new Promise((resolve) => {
                this.channelReadyPromiseResolve = resolve;
            });
        }
    }
    channelReady() {
        if (this.channelReadyPromise && this.channelReadyPromiseResolve) {
            this.channelReadyPromiseResolve();
            this.channelReadyPromise = undefined;
        }
    }
    getMaxConnectionAttempts() {
        return this.clientConfig.maxConnectionAttempts || RabbitmqConfig_1.DEFAULT_MAX_CONNECTION_ATTEMPTS;
    }
    /**
     * Connect to RabbitMQ broker
     */
    async connect() {
        if (this.connection) {
            this.connection.removeAllListeners();
            try {
                await this.connection.close();
            }
            catch (e) {
                // Do nothing... we tried to play nice
            }
        }
        const newConnection = await amqplib_1.connect(this.clientConfig);
        newConnection.on('close', () => { this.handleConnectionClosed(); });
        newConnection.on('error', (err) => { this.handleConnectionError(err); });
        this.connection = newConnection;
        this.logger.info('Connection established');
        await this.createChannel();
    }
    handleConnectionError(err) {
        if (err) {
            this.logger.error(err, 'Connection error. Ignoring');
        }
        else {
            this.logger.error('Connection error. Ignoring');
        }
    }
    async handleConnectionClosed() {
        if (!this.closed) {
            // We haven't been explicitly closed, so we should reopen the channel
            this.logger.warn('Connection was closed unexpectedly... reconnecting');
            let attempts = 0;
            while (attempts < this.getMaxConnectionAttempts()) {
                try {
                    attempts++;
                    await this.connect();
                    return;
                }
                catch (e) {
                    this.logger.warn(e, 'Failed reconnection attempt');
                }
            }
            this.logger.error(`Couldn't reconnect after ${this.getMaxConnectionAttempts()} attempts`);
            // tslint:disable-next-line
            if (this.clientConfig.exitOnIrrecoverableReconnect !== false) {
                this.logger.error('Cowardly refusing to continue. Calling shutdown function');
                this.shutdownFunction();
            }
        }
    }
    async createChannel() {
        if (this.channel) {
            this.channel.removeAllListeners();
            try {
                await this.channel.close();
            }
            catch (e) {
                // Do nothing... we tried to play nice
            }
        }
        const newChannel = await this.connection.createChannel();
        newChannel.on('close', (err) => { this.handleChannelClosed(err); });
        newChannel.on('error', (err) => { this.handleChannelError(err); });
        this.channel = newChannel;
        this.logger.info('Channel opened');
        this.channelReady();
    }
    async recreateChannel() {
        let attempts = 0;
        while (attempts < this.getMaxConnectionAttempts()) {
            try {
                attempts++;
                await this.createChannel();
                return;
            }
            catch (e) {
                this.logger.warn(e, 'Failed channel re-creation attempt');
                if (connection_1.isFatalError(e)) {
                    this.logger.warn('Cannot recreate channel on closed connection');
                    return;
                }
            }
        }
        this.logger.error(`Couldn't re-create channel after ${this.getMaxConnectionAttempts()} attempts`);
        // tslint:disable-next-line
        if (this.clientConfig.exitOnIrrecoverableReconnect !== false) {
            this.logger.error('Cowardly refusing to continue. Calling shutdown function');
            this.shutdownFunction();
        }
    }
    handleChannelError(err) {
        this.logger.error(err, 'Channel error, recreating channel');
        this.recreateChannel();
    }
    handleChannelClosed(err) {
        if (!this.closed) {
            // We haven't been explicitly closed, so we should reopen the channel
            this.logger.warn('Channel was closed unexpectedly... recreating');
            this.recreateChannel();
        }
    }
    async close() {
        this.closed = true;
        await this.closeChannel();
        await this.closeConnection();
    }
    async closeChannel() {
        this.channel.close();
        this.channel.removeAllListeners();
    }
    async closeConnection() {
        this.connection.close();
        this.connection.removeAllListeners();
    }
}
exports.RabbitmqClient = RabbitmqClient;
//# sourceMappingURL=RabbitmqClient.js.map